@@ -15,6 +15,7 @@
 use crate as cincinnati;
 
 use self::cincinnati::plugins::internal::graph_builder::release::Metadata;
+use self::cincinnati::plugins::internal::graph_builder::release::MetadataKind;
 use self::cincinnati::plugins::prelude_plugin_impl::*;
 
 use flate2::read::GzDecoder;
@@ -22,6 +23,7 @@ use futures::lock::Mutex as FuturesMutex;
 use futures::prelude::*;
 use futures::TryStreamExt;
 use log::{debug, error, trace, warn};
+use semver::Version;
 use serde::Deserialize;
 use serde_json;
 use std::fs::File;
@@ -60,7 +62,7 @@ pub mod cache {
     }
 }
 
-#[derive(Debug, Clone, PartialEq, Default)]
+#[derive(Debug, Clone, PartialEq, Default, Deserialize)]
 pub struct Registry {
     pub(crate) scheme: String,
     pub(crate) insecure: bool,
@@ -198,18 +200,13 @@ pub fn read_credentials(
     })
 }
 
-/// Fetches a vector of all release metadata from the given repository, hosted on the given
-/// registry.
-pub async fn fetch_releases(
+pub async fn new_registry_client(
     registry: &Registry,
     repo: &str,
     username: Option<&str>,
     password: Option<&str>,
-    cache: cache::Cache,
-    manifestref_key: &str,
-    concurrency: usize,
-) -> Result<Vec<cincinnati::plugins::internal::graph_builder::release::Release>, Error> {
-    let registry_client = {
+) -> Result<dkregistry::v2::Client, Error> {
+    let client = {
         let client_builder = dkregistry::v2::Client::configure()
             .registry(&registry.host_port_string())
             .insecure_registry(registry.insecure);
@@ -238,6 +235,22 @@ pub async fn fetch_releases(
         }
     };
 
+    Ok(client)
+}
+
+/// Fetches a vector of all release metadata from the given repository, hosted on the given
+/// registry.
+pub async fn fetch_releases(
+    registry: &Registry,
+    repo: &str,
+    username: Option<&str>,
+    password: Option<&str>,
+    cache: cache::Cache,
+    manifestref_key: &str,
+    concurrency: usize,
+) -> Result<Vec<cincinnati::plugins::internal::graph_builder::release::Release>, Error> {
+    let registry_client = new_registry_client(registry, repo, username, password).await?;
+
     let registry_client_get_tags = registry_client.clone();
     let tags = Box::pin(get_tags(repo, &registry_client_get_tags).await);
 
@@ -366,6 +379,15 @@ async fn lookup_or_fetch(
             cached_metadata.clone()
         }
         None => {
+            let placeholder = Option::from(Metadata {
+                kind: MetadataKind::V0,
+                version: Version::new(0, 0, 0),
+                previous: vec![],
+                next: vec![],
+                metadata: Default::default(),
+            });
+            cache.write().await.insert(manifestref.clone(), placeholder);
+
             let metadata = find_first_release_metadata(
                 layer_digests,
                 registry_client,
@@ -593,7 +615,3 @@ mod tests {
         }
     }
 }
-
-#[cfg(test)]
-#[cfg(feature = "test-net")]
-mod network_tests;
